﻿namespace Shapes
{
    /// <summary>
    /// Utility class for shape operations.
    /// </summary>
    public static class ShapeUtils
    {
        /// <summary>
        /// Calculate the area of a given shape.
        /// </summary>
        /// <param name="shape">The shape whose area will be calculated.</param>
        /// <returns>The area of the given shape.</returns>
        public static double CalculateArea(IShape shape) => shape.CalculateArea();
    }

    /// <summary>
    /// Interface for geometric shapes that can calculate their area.
    /// </summary>
    public interface IShape
    {
        /// <summary>
        /// Calculate the area of the shape.
        /// </summary>
        /// <returns>The area of the shape.</returns>
        double CalculateArea();
    }

    /// <summary>
    /// Represents a circle.
    /// </summary>
    public class Circle : IShape
    {
        /// <summary>
        /// Gets the radius of the circle.
        /// </summary>
        public double Radius { get; }

        /// <summary>
        /// Initialize a new instance of the Circle class with the given radius.
        /// </summary>
        /// <param name="radius">The radius of the circle.</param>
        public Circle(double radius)
        {
            Radius = radius;
        }

        /// <summary>
        /// Calculate the area of the circle.
        /// </summary>
        /// <returns>The area of the circle.</returns>
        public double CalculateArea() => Math.PI * Math.Pow(Radius, 2);
    }

    /// <summary>
    /// Represents a triangle.
    /// </summary>
    public class Triangle : IShape
    {
        /// <summary>
        /// Gets the length of side A.
        /// </summary>
        public double A { get; }

        /// <summary>
        /// Gets the length of side B.
        /// </summary>
        public double B { get; }

        /// <summary>
        /// Gets the length of side C.
        /// </summary>
        public double C { get; }

        /// <summary>
        /// Initialize a new instance of the Triangle class with the given sides.
        /// </summary>
        /// <param name="a">Length of side A.</param>
        /// <param name="b">Length of side B.</param>
        /// <param name="c">Length of side C.</param>
        public Triangle(double a, double b, double c)
        {
            (A, B, C) = (a, b, c);
        }

        /// <summary>
        /// Determines if the triangle is a right-angled triangle.
        /// </summary>
        public bool IsRightAngle
        {
            get
            {
                var sides = new[] { A, B, C };
                Array.Sort(sides);
                return Math.Abs(Math.Pow(sides[0], 2) + Math.Pow(sides[1], 2) - Math.Pow(sides[2], 2)) < 1e-10;
            }
        }
        
        /// <summary>
        /// Calculate the area of the triangle.
        /// </summary>
        /// <returns>The area of the triangle.</returns>
        public double CalculateArea()
        {
            var s = (A + B + C) / 2;
            return Math.Sqrt(s * (s - A) * (s - B) * (s - C));
        }
    }
}